#include "matrix-math.h"
#include <math.h>
#include <stdio.h>

/**
 * overwrites a 4x4 matrix with the identity matrix
 */
void mat4Identity(mat4 mat) {
	for (int i = 0; i < 16; i++) {
		mat[i] = (i % 4 == i / 4) ? 1 : 0;
	}
}

/**
 * copies a mat4 from src to dst
 */
void mat4Copy(mat4 src, mat4 dst) {
	for (int i = 0; i < 16; i++) {
		dst[i] = src[i];
	}
}

/**
 * sets all the values in a mat4 to zero
 */
void mat4Empty(mat4 mat) {
	for (int i = 0; i < 16; i++) {
		mat[i] = 0;
	}
}

/**
 * mutliplies A with B and stores the result in results
 */
void mat4Multiply(mat4 result, mat4 A, mat4 B) {
	// if result is one of the arguments, modify copy instead of result directly
	if (result == A || result == B) {
		mat4 tempResult;
		mat4Multiply(tempResult, A, B);
		mat4Copy(tempResult, result);
		return;
	}

	for (int i = 0; i < 16; i++) {
		int col = i / 4;
		int row = i % 4;
		GLfloat* curr = &(result[i]);

		*curr = 0;

		for (int j = 0; j < 4; j++) {
			*curr += A[row + j * 4] * B[j + col * 4];
		}
	}
}

/**
 * prints a mat4 to the screen
 */
void mat4Print(mat4 m) {
	for (int i = 0; i < 4; i++) {
		for (int j = 0; j < 4; j++) {
			printf("%.2f ", m[j * 4 + i]);
		}
		printf("\n");
	}
}